use std::error::Error;
use std::fs;
use std::path::{PathBuf};
use crate::manifest_file::ManifestFile;

mod manifest_file;

#[derive(Debug)]
pub struct Manifest {
    directory: PathBuf,
}

impl Manifest {
    pub fn from(path: &PathBuf) -> Manifest {
        let mut directory: PathBuf = path.into();

        if path.is_file() {
            directory = path.parent()
                .map(|parent| parent.to_path_buf())
                .unwrap_or_else(|| path.into())
        }

        if let Err(_err) = fs::create_dir_all(&directory) {
            panic!("Failed to created directory: {:?}", &directory);
        }

        Manifest { directory: directory.canonicalize().unwrap() }
    }

    pub fn init(&self) -> bool {
        let file: PathBuf = self.file_path();

        if !file.exists() {
            let manifest_file: ManifestFile = ManifestFile::from(self.directory_name());

            return match manifest_file.write(&file) {
                Ok(_) => true,
                Err(_) => panic!("Unable to write to file: {:?}", &file)
            }
        }

        false
    }

    pub fn file_path(&self) -> PathBuf  {
        return PathBuf::from(format!("{}/{}", self.directory.to_string_lossy(), "mccli.yaml"));
    }

    pub fn directory_path(&self) -> PathBuf {
        return PathBuf::from(&self.directory);
    }

    pub fn directory_name(&self) -> String {
        let directory = self.directory_path();

        return directory
            .file_name()
            .unwrap()
            .to_string_lossy()
            .to_string();
    }
}
