use std::error::Error;
use std::fs;
use std::fs::File;
use serde::{Serialize, Deserialize};
use std::path::PathBuf;
use serde::de::DeserializeOwned;

#[derive(Serialize, Deserialize, Debug, Default)]
pub struct ManifestFile {
    name: String,
}

impl ManifestFile {
    pub fn from(name: String) -> ManifestFile {
        return ManifestFile { name }
    }

    pub fn read<Content: DeserializeOwned>(path: &PathBuf) -> Result<Content, Box<dyn Error>> {
        let file = File::open(path)?;

        let data: Content = serde_yaml::from_reader(file)?;

        Ok(data)
    }

    pub fn write(&self, path: &PathBuf) -> Result<bool, Box<dyn Error>> {
        if path.exists() {
            fs::remove_file(path)?;
        }

        let file = File::create(path)?;

        serde_yaml::to_writer(file, self)?;

        Ok(true)
    }
}